#import time
from machine import Pin, ADC
import aioble
import uasyncio as asyncio
import bluetooth
from micropython import const

_ADVERTISING_INTERVAL_US = const(200_000)
_APPEARANCE = const(0x0552) # Multi-sensor
_TEMP_MEASUREMENT_INTERVAL_MS = const(15_000)
_TEMP_MEASUREMENT_FREQUENCY = const(15_000)

connected=False

async def task_peripheral():
  """ Task to handle advertising and connections """
  global connected
  while True:
    connected = False
    async with await aioble.advertise(
        _ADVERTISING_INTERVAL_US,
        appearance=_APPEARANCE,
        name='RP2-SENSOR',
        services=[_SVC_DEVICE_INFO]
    ) as connection:
      print("Connected from ", connection.device)
      connected = True

async def task_flash_led():
  """ Blink the on-board LED, faster if disconnected and slower if connected  """
  BLINK_DELAY_MS_FAST = const(100)
  BLINK_DELAY_MS_SLOW = const(500)
  while True:
    led.toggle()
    if connected:
      await asyncio.sleep_ms(BLINK_DELAY_MS_SLOW)
    else:
      await asyncio.sleep_ms(BLINK_DELAY_MS_FAST)

async def task_sensor():
  """ Task to handle sensor measures """
  while True:
    temperature =  27.0 - ((adc.read_u16() * 3.3 / 65535) - 0.706) / 0.001721
    print("T: {}°C".format(temperature))
    await asyncio.sleep_ms(_TEMP_MEASUREMENT_FREQUENCY)
    #time.sleep_ms(_TEMP_MEASUREMENT_INTERVAL_MS)

led = Pin('LED', Pin.OUT)
adc = ADC(4)

# Constants for the device information service
_SVC_DEVICE_INFO = bluetooth.UUID(0x180A)
svc_dev_info = aioble.Service(_SVC_DEVICE_INFO)

_CHAR_MANUFACTURER_NAME_STR = bluetooth.UUID(0x2A29)
_CHAR_MODEL_NUMBER_STR = bluetooth.UUID(0x2A24)
_CHAR_SERIAL_NUMBER_STR = bluetooth.UUID(0x2A25)
_CHAR_FIRMWARE_REV_STR = bluetooth.UUID(0x2A26)
_CHAR_HARDWARE_REV_STR = bluetooth.UUID(0x2A27)

# Create characteristics for device info
aioble.Characteristic(svc_dev_info, _CHAR_MANUFACTURER_NAME_STR, read=True, initial='Njord')
aioble.Characteristic(svc_dev_info, _CHAR_MODEL_NUMBER_STR, read=True, initial='J-0001')
aioble.Characteristic(svc_dev_info, _CHAR_SERIAL_NUMBER_STR, read=True, initial='J-0001-0000')
aioble.Characteristic(svc_dev_info, _CHAR_FIRMWARE_REV_STR, read=True, initial='0.0.1')
aioble.Characteristic(svc_dev_info, _CHAR_HARDWARE_REV_STR, read=True, initial='0.0.1')

aioble.register_services(svc_dev_info)


async def main():
  """ Create all the tasks """
  tasks = [
    asyncio.create_task(task_peripheral()),
    asyncio.create_task(task_flash_led()),
    asyncio.create_task(task_sensor()),
  ]
  asyncio.gather(*tasks)

  asyncio.run(main())